@@ -50,7 +50,6 @@ static GPid      tap_backend_run                        (const gchar *action,
                                                          GList       *content_types,
                                                          GtkWidget   *window,
                                                          GError     **error);
-static void      tap_backend_setup_display_cb           (gpointer data);
 
 
 
@@ -91,7 +90,7 @@ tap_backend_mime_ask (GList     *mime_applications,
   gtk_widget_show (grid);
 
   /* add the header image */
-  image = gtk_image_new_from_icon_name ("gnome-package", GTK_ICON_SIZE_DIALOG);
+  image = gtk_image_new_from_icon_name ("package-x-generic", GTK_ICON_SIZE_DIALOG);
   gtk_grid_attach (GTK_GRID (grid), image, 0, 0, 1, 1);
   gtk_widget_show (image);
 
@@ -224,8 +223,7 @@ tap_backend_mime_applications (GList *content_types)
             }
 
           /* release the list of applications for this mime type */
-          g_list_foreach (list, (GFunc) g_object_unref, NULL);
-          g_list_free (list);
+          g_list_free_full (list, g_object_unref);
         }
 
       /* check if the set is still not empty */
@@ -311,7 +309,7 @@ tap_backend_mime_application (GList     *content_types,
       if (G_LIKELY (lp == NULL))
         {
           /* use the first available archive manager */
-          mime_application = g_object_ref (G_OBJECT (mime_applications->data));
+          mime_application = G_APP_INFO (g_object_ref (G_OBJECT (mime_applications->data)));
         }
       else
         {
@@ -338,8 +336,7 @@ tap_backend_mime_application (GList     *content_types,
         }
 
       /* cleanup */
-      g_list_foreach (mime_applications, (GFunc) g_object_unref, NULL);
-      g_list_free (mime_applications);
+      g_list_free_full (mime_applications, g_object_unref);
     }
 
   return mime_application;
@@ -394,6 +391,7 @@ tap_backend_run (const gchar *action,
   GdkScreen                *screen;
   gchar                    *wrapper;
   gchar                   **argv;
+  gchar                   **envp;
   gchar                    *uri;
   GList                    *lp;
   GPid                      pid = -1;
@@ -439,17 +437,24 @@ tap_backend_run (const gchar *action,
               g_free (uri);
             }
 
+          envp = g_get_environ();
+
           /* determine the screen for this window */
           screen = gtk_widget_get_screen (window);
 
           if (screen != NULL)
-            display = gdk_screen_make_display_name (screen);
+            {
+              display = gdk_screen_make_display_name (screen);
+              if (display != NULL)
+                envp = g_environ_setenv(envp, "DISPLAY", display, TRUE);
+            }
 
           /* try to run the command */
-          if (!g_spawn_async (folder, argv, NULL, G_SPAWN_DO_NOT_REAP_CHILD, tap_backend_setup_display_cb, display, &pid, error))
+          if (!g_spawn_async (folder, argv, envp, G_SPAWN_DO_NOT_REAP_CHILD, NULL, NULL, &pid, error))
             pid = -1;
 
           /* cleanup */
+          g_strfreev (envp);
           g_strfreev (argv);
           g_free (display);
         }
@@ -459,19 +464,13 @@ tap_backend_run (const gchar *action,
     }
 
   /* cleanup */
-  g_list_foreach (content_types, (GFunc) g_free, NULL);
-  g_list_free (content_types);
+  g_list_free_full (content_types, g_free);
 
   return pid;
 }
 
 
 
-static void
-tap_backend_setup_display_cb (gpointer data)
-{
-  g_setenv ("DISPLAY", (char *) data, TRUE);
-}
 
 
 
